import { Socket, Server } from 'socket.io'
import { TSocket } from '#utilities/types'
import { Character } from '@prisma/client'
import CharacterRepository from '#repositories/characterRepository'
import { gameLogger } from '#utilities/logger'

export default class CharacterListEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('character:list', this.handleCharacterList.bind(this))
  }

  private async handleCharacterList(data: any): Promise<void> {
    try {
      const characters: Character[] = (await CharacterRepository.getByUserId(this.socket.userId!)) as Character[]
      this.socket.emit('character:list', characters)
    } catch (error: any) {
      gameLogger.error('character:list error', error.message)
    }
  }
}
